@@ -52,9 +52,9 @@ namespace Do
 {
 	internal static class AssemblyInfo
 	{
-		public const string DisplayVersion = "0.8.2";
-		public const string VersionDetails = "bzr trunk r1261";
-		public const string LocaleDirectory = "/usr/local/share/locale";
+		public const string DisplayVersion = "0.8.3.1";
+		public const string VersionDetails = "bzr core-0.8.2 r1281";
+		public const string LocaleDirectory = "/home/raof/.local/share/locale";
 	}
 }
 
