@@ -34,12 +34,14 @@ namespace Docky.Interface
 
 		bool Interruptable { get; }
 		
-		int MinimumWidth { get; }
+		int Width { get; }
 		
 		void Paint (Context cr, Gdk.Rectangle dockArea, Gdk.Point cursor);
 
 		void Clicked (Gdk.Rectangle dockArea, Gdk.Point cursor);
 
+		void Scrolled (Gdk.ScrollDirection direction);
+
 		void Interrupt ();
 	}
 }
