@@ -18,6 +18,7 @@
 //
 
 using System;
+using System.IO;
 using System.Reflection;
 
 using NDesk.DBus;
@@ -174,6 +175,13 @@ namespace Do.Platform.Linux
 					autostartfile = DesktopItem.NewFromFile (System.IO.Path.Combine (AssemblyInfo.InstallData, "applications/gnome-do.desktop"),
 					                                         DesktopItemLoadFlags.NoTranslations);
 					try {
+						if (!Directory.Exists (AutoStartDir))
+							Directory.CreateDirectory (AutoStartDir);
+
+						// This *enables* autostart, by setting the "Hidden" key (which disables autostart) to false.
+						// Explicitly setting this key fixes LP #398303; otherwise our IsAutoStartEnabled method won't find
+						// the AutoStartKey, and will erroneously return false.
+						autostartfile.SetBoolean (AutoStartKey, false);
 						autostartfile.Save (AutoStartUri, true);
 						autostartfile.Location = AutoStartUri;
 					} catch (Exception e) {
