@@ -36,7 +36,7 @@ namespace Do
 		const string StartAtLoginKey = "StartAtLogin";
 		const string AlwaysShowResultsKey = "AlwaysShowResults";
 
-		const string ThemeDefaultValue = "Classic Interface";
+		const string ThemeDefaultValue = "Classic";
 		const bool QuietStartDefaultValue = false;
 		const bool StartAtLoginDefaultValue = false;
 		const bool AlwaysShowResultsDefaultValue = false;
