@@ -48,6 +48,12 @@ namespace Do.Core
 
 		public event EventHandler Summoned;
 
+		internal IDoWindow Window {
+			get {
+				return window;
+			}
+		}
+		
 		public Gtk.AboutDialog AboutDialog { get; private set; }
 		public PreferencesWindow PreferencesWindow { get; private set; }
 		
@@ -86,6 +92,7 @@ namespace Do.Core
 			im_context.UsePreedit = false;
 			im_context.Commit += OnIMCommit;
 			im_context.FocusIn ();
+			
 		}
 		
 		void OnIMCommit (object sender, Gtk.CommitArgs e)
@@ -98,6 +105,99 @@ namespace Do.Core
 		{
 			SetTheme (Do.Preferences.Theme);
 			Do.Preferences.ThemeChanged += OnThemeChanged;
+			Screen.Default.CompositedChanged += OnCompositingChanged;
+			
+			// Register Shortcuts
+			// TODO: Localize the text here.
+			// Previous shortcuts
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("SummonKey", 
+						Catalog.GetString ("Summon GNOME Do"), 
+						OnSummonKeyPressEvent),
+					"<Super>space");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("TextModeKey",
+						Catalog.GetString ("Enter text mode"),
+						OnTextModePressEvent),
+					"period");
+			// New shortcuts
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("CopyKey",
+						Catalog.GetString ("Copy Text"),
+						OnCopyEvent),
+					"<Control>c");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("PasteKey",
+						Catalog.GetString ("Paste Text"),
+						OnPasteEvent),
+					"<Control>v");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("AlternateEscapeKey",
+						Catalog.GetString ("Alternate Escape"),
+						OnEscapeKeyPressEvent));
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("AlternateActivateKey",
+						Catalog.GetString ("Alternate Activate"),
+						OnActivateKeyPressEvent));
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("AlternateDeleteKey",
+						Catalog.GetString ("Alternate Delete"),
+						OnDeleteKeyPressEvent));
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("ShiftTabKey",
+						Catalog.GetString ("Previous Pane"),
+						OnShiftTabKeyPressEvent),
+					"ISO_Left_Tab");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("TabKey",
+						Catalog.GetString ("Next Pane"),
+						OnTabKeyPressEvent),
+					"Tab");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("UpKey",
+						Catalog.GetString ("Previous Result"),
+						OnUpKeyPressEvent),
+					"Up");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("DownKey",
+						Catalog.GetString ("Next Result"),
+						OnDownKeyPressEvent),
+					"Down");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("HomeKey",
+						Catalog.GetString ("First Result"),
+						OnHomeKeyPressEvent),
+					"Home");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("EndKey",
+						Catalog.GetString ("Last Result"),
+						OnEndKeyPressEvent),
+					"End");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("PageUpKey",
+						Catalog.GetString ("Previous 5 Results"),
+						OnPageUpKeyPressEvent),
+					"Page_Up");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("PageDownKey",
+						Catalog.GetString ("Next 5 Results"),
+						OnPageDownKeyPressEvent),
+					"Page_Down");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("LeftKey",
+						Catalog.GetString ("Step out of Item"),
+						OnLeftKeyPressEvent),
+					"Left");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("RightKey",
+						Catalog.GetString ("Browse Into Item"),
+						OnRightKeyPressEvent),
+					"Right");
+			Do.Keybindings.RegisterShortcut (
+					new Shortcut ("CommaKey",
+						Catalog.GetString ("Multiple selection"),
+						OnSelectionKeyPressEvent),
+					"comma");
 		}
 
 		void OnSummoned ()
@@ -106,6 +206,12 @@ namespace Do.Core
 			Summoned (this, EventArgs.Empty);
 		}
 		
+		void OnCompositingChanged(object sender, EventArgs args)
+		{
+			UnsetTheme();
+			SetTheme(Do.Preferences.Theme);
+		}
+		
 		void OnThemeChanged (object sender, PreferencesChangedEventArgs e)
 		{
 			string oldTheme = e.OldValue as string, newTheme = Do.Preferences.Theme;
@@ -203,6 +309,68 @@ namespace Do.Core
 			}
 		}
 		
+		Pane WorkingActionPane {
+			get {
+				Item first, second;
+				first = GetSelection (Pane.First);
+				second = GetSelection (Pane.Second);
+				
+				if (first != null && second != null) {
+					if (first.IsAction () && first.AsAction ().Safe.SupportsItem (second)) {
+						return Pane.First;
+					} else if (second.IsAction () && second.AsAction ().Safe.SupportsItem (first)) {
+						return Pane.Second;
+					}
+				}
+				return Pane.None;
+			}
+		}
+		
+		Pane WorkingItemPane {
+			get {
+				Item first, second;
+				first = GetSelection (Pane.First);
+				second = GetSelection (Pane.Second);
+				
+				if (first != null && second != null) {
+					if (first.IsAction () && first.AsAction ().Safe.SupportsItem (second)) {
+						return Pane.Second;
+					} else if (second.IsAction () && second.AsAction ().Safe.SupportsItem (first)) {
+						return Pane.First;
+					}
+				}
+				return Pane.None;
+			}
+		}
+		
+		Act WorkingAction {
+			get {
+				return GetSelection (WorkingActionPane).AsAction ();
+			}
+		}
+		
+		Item WorkingItem {
+			get {
+				return GetSelection (WorkingItemPane);
+			}
+		}
+		
+		IEnumerable<Item> WorkingItems {
+			get {
+				Pane workingPane = WorkingItemPane;
+				if (workingPane == Pane.None)
+					return null;
+				
+				return controllers [(int) workingPane].FullSelection;
+			}
+		}
+		
+		IEnumerable<Item> WorkingModItems {
+			get {
+				return controllers [(int) Pane.Third].FullSelection;
+			}
+		}
+		
 		/// <summary>
 		/// Sets/Unsets third pane visibility if possible.
 		/// </summary>
@@ -239,12 +407,8 @@ namespace Do.Core
 		/// </summary>
 		bool ThirdPaneAllowed {
 			get {
-				Act action;
-				Element first, second;
-
-				first = GetSelection (Pane.First);
-				second = GetSelection (Pane.Second);
-				action = first as Act ?? second as Act;
+				Act action = WorkingAction;
+				
 				return action != null &&
 					action.SupportedModifierItemTypes.Any () &&
 					controllers [1].Results.Any ();
@@ -257,14 +421,9 @@ namespace Do.Core
 		/// </value>
 		bool ThirdPaneRequired {
 			get {
-				Item item;
-				Act action;
-				Element first, second;
-
-				first = GetSelection (Pane.First);
-				second = GetSelection (Pane.Second);
-				action = first as Act ?? second as Act;
-				item = first as Item ?? second as Item;
+				Item item = WorkingItem;
+				Act action = WorkingAction;
+				
 				return action != null && item != null &&
 					action.SupportedModifierItemTypes.Any () &&
 					!action.ModifierItemsOptional &&
@@ -289,9 +448,9 @@ namespace Do.Core
 		/// Summons a window with elements in it... seems to work
 		/// </summary>
 		/// <param name="elements">
-		/// A <see cref="Element"/>
+		/// A <see cref="Item"/>
 		/// </param>
-		public void SummonWithElements (IEnumerable<Element> elements)
+		public void SummonWithItems (IEnumerable<Item> elements)
 		{
 			if (!IsSummonable) return;
 			
@@ -319,8 +478,8 @@ namespace Do.Core
 		{
 			return controllers [(int) pane].TextType == TextModeType.Explicit ||
 				controllers [(int) pane].TextType == TextModeType.ExplicitFinalized;
-		}
-		
+		}	
+
 #region KeyPress Handling
 
 		Key IfVertical (Key ifKey, Key elseKey)
@@ -337,59 +496,29 @@ namespace Do.Core
 		{
 			Key key = (Key) evnt.KeyValue;
 
-			// User set keybindings
-			if (KeyEventToString (evnt).Equals (Do.Preferences.SummonKeybinding)) {
-				OnSummonKeyPressEvent (evnt);
-				return;
-			} 
 			
-			if (KeyEventToString (evnt).Equals (Do.Preferences.TextModeKeybinding)) {
-				OnTextModePressEvent (evnt);
-				return;
-			}
-			
-			// Check for paste
-			if ((evnt.State & ModifierType.ControlMask) != 0) {
-				if (evnt.Key == Key.v) {
-					OnPasteEvent ();
-					return;
-				}
-				if (evnt.Key == Key.c) {
-					OnCopyEvent ();
-					return;
-				}
-			}
-
+			// Currently - only hardcoded are enter keys, escape and delete/backspace
 			if (key == Key.Escape) {
 				OnEscapeKeyPressEvent (evnt);
 			} else if (key == Key.Return ||
-			           key == Key.ISO_Enter ||
-			           key == Key.KP_Enter) {
+				   key == Key.ISO_Enter ||
+				   key == Key.KP_Enter) {
 				OnActivateKeyPressEvent (evnt);
 			} else if (key == Key.Delete ||
-			           key == Key.BackSpace) {
+				   key == Key.BackSpace) {
 				OnDeleteKeyPressEvent (evnt);
-			} else if (key == Key.Tab ||
-			           key == Key.ISO_Left_Tab) {
-				OnTabKeyPressEvent (evnt);
-			} else if (key == UpKey ||
-			           key == DownKey ||
-			           key == Key.Home ||
-			           key == Key.End ||
-			           key == Key.Page_Up ||
-			           key == Key.Page_Down) {
-				OnUpDownKeyPressEvent (evnt);
-			} else if (key == RightKey ||
-			           key == LeftKey) {
-				OnRightLeftKeyPressEvent (evnt);
-			} else if (key == Key.comma) {
-				OnSelectionKeyPressEvent (evnt);
 			} else {
+				// User set keybindings
+				Shortcut sc = Do.Keybindings.GetShortcutByKeycode (KeyEventToString (evnt));
+				if (sc != null) {
+					sc.Callback (evnt);
+					return;
+				}
 				OnInputKeyPressEvent (evnt);
 			}
 		}
 		
-		void OnPasteEvent ()
+		void OnPasteEvent (EventKey evnt)
 		{
 			Gtk.Clipboard clip = Gtk.Clipboard.Get (Selection.Clipboard);
 			if (!clip.WaitIsTextAvailable ()) {
@@ -399,11 +528,10 @@ namespace Do.Core
 			SearchController.SetString (SearchController.Query + str);
 		}
 		
-		void OnCopyEvent ()
+		void OnCopyEvent (EventKey evnt)
 		{
-			Gtk.Clipboard clip = Gtk.Clipboard.Get (Selection.Clipboard);
-			if (SearchController.Selection != null)
-				clip.Text = SearchController.Selection.Name;
+			if (SearchController.Selection is Item)
+				Services.Environment.CopyToClipboard (SearchController.Selection);
 		}
 		
 		void OnActivateKeyPressEvent (EventKey evnt)
@@ -494,28 +622,32 @@ namespace Do.Core
 			}
 		}
 		
-		void OnRightLeftKeyPressEvent (EventKey evnt)
+		void OnLeftKeyPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			if (!SearchController.Results.Any ()) return;
 
-			if ((Key) evnt.KeyValue == RightKey) {
-				// We're attempting to browse the contents of an item, so increase its
-				// relevance.
-				SearchController.Selection
-					.IncreaseRelevance (SearchController.Query, null);
-				if (SearchController.ItemChildSearch ()) GrowResults ();
-			} else if ((Key) evnt.KeyValue == LeftKey) {
-				// We're attempting to browse the parent of an item, so decrease its
-				// relevance. This makes it so we can merely visit an item's children,
-				// and navigate back out of the item, and leave that item's relevance
-				// unchanged.
-				SearchController.Selection
-					.DecreaseRelevance (SearchController.Query, null);
-				if (SearchController.ItemParentSearch ()) GrowResults ();
-			}
+			// We're attempting to browse the parent of an item, so decrease its
+			// relevance. This makes it so we can merely visit an item's children,
+			// and navigate back out of the item, and leave that item's relevance
+			// unchanged.
+			SearchController.Selection
+				.DecreaseRelevance (SearchController.Query, null);
+			if (SearchController.ItemParentSearch ()) GrowResults ();
 		}
-		
+
+		// Hmm.
+		void OnRightKeyPressEvent (EventKey evnt)
+		{
+			im_context.Reset ();
+			if (!SearchController.Results.Any ()) return;
+
+			// We're attempting to browse the contents of an item, so increase its
+			// relevance.
+			SearchController.Selection.IncreaseRelevance (SearchController.Query, null);
+			if (SearchController.ItemChildSearch ()) GrowResults ();
+		}
+
 		void OnTabKeyPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
@@ -523,15 +655,24 @@ namespace Do.Core
 
 			if (SearchController.TextType == TextModeType.Explicit) {
 				SearchController.FinalizeTextMode ();
-				UpdatePane (CurrentPane);
 			}
 				
-			if (evnt.Key == Key.Tab) {
-				NextPane ();
-			} else if (evnt.Key == Key.ISO_Left_Tab) {
-				PrevPane ();
-			}
+			NextPane ();
+			UpdatePane (CurrentPane);
 		}
+
+		void OnShiftTabKeyPressEvent (EventKey evnt)
+		{
+			im_context.Reset ();
+			ShrinkResults ();
+
+			if (SearchController.TextType == TextModeType.Explicit) {
+				SearchController.FinalizeTextMode ();
+			}
+			PrevPane ();
+			UpdatePane (CurrentPane);
+		}
+			
 		
 		void OnTextModePressEvent (EventKey evnt)
 		{
@@ -548,38 +689,56 @@ namespace Do.Core
 			UpdatePane (CurrentPane);
 		}
 		
-		void OnUpDownKeyPressEvent (EventKey evnt)
+		void OnUpKeyPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
-			if (evnt.Key == UpKey) {
-				if (!results_grown) {
-					if (SearchController.Cursor > 0)
-						GrowResults ();
-					return;
-				} else {
-					if (SearchController.Cursor <= 0) {
-						ShrinkResults ();
-						return;
-					}
-					SearchController.Cursor--;
-                }
-			} else if (evnt.Key == DownKey) {
-				if (!results_grown) {
+			if (!results_grown) {
+				if (SearchController.Cursor > 0)
 					GrowResults ();
+				return;
+			} else {
+				if (SearchController.Cursor <= 0) {
+					ShrinkResults ();
 					return;
 				}
-				SearchController.Cursor++;
-			} else if (evnt.Key == Key.Home) {
-				SearchController.Cursor = 0;
-			} else if (evnt.Key == Key.End) {
-				SearchController.Cursor = SearchController.Results.Count - 1;
-			} else if (evnt.Key == Key.Page_Down) {
-				SearchController.Cursor += 5;
-			} else if (evnt.Key == Key.Page_Up) {
-				SearchController.Cursor -= 5;
+				SearchController.Cursor--;
 			}
 		}
+
+		void OnDownKeyPressEvent (EventKey evnt)
+		{
+			im_context.Reset ();
+			if (!results_grown) {
+				GrowResults ();
+				return;
+			}
+			SearchController.Cursor++;
+		}
 		
+		void OnHomeKeyPressEvent (EventKey evnt)
+		{
+			im_context.Reset ();
+			SearchController.Cursor = 0;
+		}
+
+		void OnEndKeyPressEvent (EventKey evnt)
+		{
+			im_context.Reset ();
+			SearchController.Cursor = SearchController.Results.Count - 1;
+		}
+
+		void OnPageUpKeyPressEvent (EventKey evnt)
+		{
+			im_context.Reset ();
+			SearchController.Cursor -= 5;
+		}
+		
+		void OnPageDownKeyPressEvent (EventKey evnt)
+		{
+			im_context.Reset ();
+			SearchController.Cursor += 5;
+		}
+				
 		/// <summary>
 		/// Converts a keypress into a human readable string for comparing
 		/// against values in GConf.
@@ -651,7 +810,7 @@ namespace Do.Core
 		/// This method determines what to do when a search is completed and takes the appropriate action
 		/// </summary>
 		/// <param name="o">
-		/// A <see cref="System.Element"/>
+		/// A <see cref="System.Item"/>
 		/// </param>
 		/// <param name="state">
 		/// A <see cref="SearchFinishState"/>
@@ -749,9 +908,9 @@ namespace Do.Core
 			results_grown = false;
 		}
 		
-		Element GetSelection (Pane pane)
+		Item GetSelection (Pane pane)
 		{
-			Element o;
+			Item o;
 
 			try {
 				o = controllers [(int) pane].Selection;
@@ -764,72 +923,58 @@ namespace Do.Core
 		void PerformAction (bool vanish)
 		{
 			Act action;
-			Element first, second, third;
 			string actionQuery, itemQuery, modItemQuery;
-			ICollection<Item> items, modItems;
+			IEnumerable<Item> items, modItems;
 
 			if (vanish) Vanish ();
 			// Flush main thread queue to get Vanish to complete.
 			Services.Application.FlushMainThreadQueue ();
 
-			items = new List<Item> ();
-			modItems = new List<Item> ();
-			first  = GetSelection (Pane.First);
-			second = GetSelection (Pane.Second);
-			third  = GetSelection (Pane.Third);
-			action = first as Act ?? second as Act;
-
+			action = WorkingAction;
+			items = WorkingItems;
+			modItems = ThirdPaneVisible ? WorkingModItems : Enumerable.Empty<Item> ();
+			
 			// If the current state of the controller is invalid, warn and return
 			// early.
-			if (first == null || second == null || action == null) {
+			if (items == null || action == null) {
 				Log<Controller>
 					.Warn ("Controller state was not valid, so the action could not be performed.");
 				if (vanish) Reset ();
 				return;
 			}
-
-			if (first is Item) {
-				foreach (Item item in controllers [0].FullSelection)
-					items.Add (item);
-				itemQuery = controllers [0].Query;
-				actionQuery = controllers [1].Query;
-			} else {
-				foreach (Item item in controllers [1].FullSelection)
-					items.Add (item);
-				itemQuery = controllers [1].Query;
-				actionQuery = controllers [0].Query;
-			}
-
-			modItemQuery = null;
-			if (third != null && ThirdPaneVisible) {
-				foreach (Item item in controllers [2].FullSelection)
-					modItems.Add (item);
-				modItemQuery = controllers [2].Query;
-			}
+			
+			actionQuery  = controllers [(int) WorkingActionPane].Query;
+			itemQuery    = controllers [(int) WorkingItemPane].Query;
+			modItemQuery = controllers [(int) Pane.Third].Query;
+			
+			if (ThirdPaneVisible)
+				modItemQuery = controllers [(int) Pane.Third].Query;
 
 			/////////////////////////////////////////////////////////////
 			/// Relevance accounting
 			/////////////////////////////////////////////////////////////
 			
-			if (first is Item) {
+			if (WorkingActionPane == Pane.Second) {
 				// Act is in second pane.
 				// Increase the relevance of the items.
-				foreach (Element item in items)
+				foreach (Item item in items)
 					item.IncreaseRelevance (itemQuery, null);
 
 				// Increase the relevance of the action /for each item/:
-				foreach (Element item in items)
+				foreach (Item item in items)
 					action.IncreaseRelevance (actionQuery, item);
 			} else {
 				// Act is in first pane.
 				// Increase the relevance of each item for the action.
-				foreach (Element item in items)
+				foreach (Item item in items)
 					item.IncreaseRelevance (itemQuery, action);
 				action.IncreaseRelevance (actionQuery, null);
 			}
 
-			if (third != null && ThirdPaneVisible)
-				third.IncreaseRelevance (modItemQuery, action);
+			if (ThirdPaneVisible) {
+				foreach (Item item in modItems)
+					item.IncreaseRelevance (modItemQuery, action);
+			}
 
 			if (vanish) Reset ();
 
@@ -839,13 +984,13 @@ namespace Do.Core
 
 		void PerformAction (Act action, IEnumerable<Item> items, IEnumerable<Item> modItems)
 		{
-			if (action == null) throw new ArgumentNullException ("action");
-			if (items == null) throw new ArgumentNullException ("items");
+			if (action == null)   throw new ArgumentNullException ("action");
+			if (items == null)    throw new ArgumentNullException ("items");
 			if (modItems == null) throw new ArgumentNullException ("modItems");
 
 			IEnumerable<Item> results = action.Safe.Perform (items, modItems);
 			if (results.Any ()) {
-				SummonWithElements (results.OfType<Element> ());
+				SummonWithItems (results);
 			}
 		}
 
@@ -876,6 +1021,7 @@ namespace Do.Core
 				PreferencesWindow = new PreferencesWindow ();
 				PreferencesWindow.Hidden += delegate {
 					// Release the window.
+					PreferencesWindow.Dispose ();
 					PreferencesWindow.Destroy ();
 					PreferencesWindow = null;
 					// Reload universe.
@@ -889,6 +1035,11 @@ namespace Do.Core
 
 		public void ShowAbout ()
 		{
+			if (AboutDialog != null) {
+				AboutDialog.GdkWindow.Raise ();
+				return;
+			}
+			
 			string logo;
 
 			Vanish ();
@@ -909,14 +1060,22 @@ namespace Do.Core
 				"applications, music, contacts, and more!");
 			AboutDialog.Website = "http://do.davebsd.com/";
 			AboutDialog.WebsiteLabel = Catalog.GetString ("Visit Homepage");
+			Gtk.AboutDialog.SetUrlHook((dialog, link) => Services.Environment.OpenUrl (link));
 			AboutDialog.IconName = "gnome-do";
 
 			if (AboutDialog.Screen.RgbaColormap != null)
 				Gtk.Widget.DefaultColormap = AboutDialog.Screen.RgbaColormap;
-
-			AboutDialog.Run ();
-			AboutDialog.Destroy ();
-			AboutDialog = null;
+			
+			AboutDialog.Response += delegate {
+				AboutDialog.Hide ();
+			};
+			
+			AboutDialog.Hidden += delegate {
+				AboutDialog.Destroy ();
+				AboutDialog = null;
+			};
+			
+			AboutDialog.Show ();
 		}
 		#endregion
 		
@@ -938,9 +1097,9 @@ namespace Do.Core
 		
 		public ControlOrientation Orientation { get; set; }
 		
-		public bool ElementHasChildren (Element element)
+		public bool ItemHasChildren (Item item)
 		{
-			return element is Item && (element as Item).HasChildren ();
+			return item.HasChildren ();
 		}
 		
 		#endregion
