@@ -25,10 +25,9 @@ using System.Text.RegularExpressions;
 using System.Collections.Generic;
 
 using Gtk;
-using Mono.Unix;
 using Mono.Addins;
-using Mono.Addins.Gui;
 using Mono.Addins.Setup;
+using Mono.Unix;
 
 using Do;
 using Do.Core;
@@ -44,8 +43,9 @@ namespace Do.UI
 	public partial class ManagePluginsPreferencesWidget : Bin, IConfigurable
 	{
 
-		const string PluginWikiPageFormat =
-			"http://do.davebsd.com/wiki/index.php?title={0}_Plugin";
+		const string WikiPage = "http://do.davebsd.com/wiki/{0}{1}";
+		const string PluginWikiPageFormat = "_Plugin";
+		const string DockletWikiPageFormat = "_Docklet";
 
 		PluginNodeView nview;
 		SearchEntry search_entry;
@@ -66,6 +66,8 @@ namespace Do.UI
 		{
 			Build ();
 			
+			PluginManager.RefreshPlugins ();
+			
 			search_entry = new SearchEntry ();
 			nview = new PluginNodeView ();
 			nview.PluginToggled += OnPluginToggled;
@@ -92,44 +94,56 @@ namespace Do.UI
 			search_entry.Show();
 			search_entry.Ready = true;
 			
-			hbox1.PackStart (search_entry,true,true,0);
-			hbox1.ShowAll();
+			hbox1.PackStart (search_entry, true, true, 0);
+			hbox1.ShowAll ();
 			
-			Services.Application.RunOnMainThread (() =>
-				search_entry.InnerEntry.GrabFocus ()
-			);
+			Services.Application.RunOnMainThread (() => search_entry.InnerEntry.GrabFocus ());
 		}
 		
 		protected void OnDragDataReceived (object sender, DragDataReceivedArgs args)
 		{
-			string data = Encoding.UTF8.GetString (args.SelectionData.Data);
+			string data;
+			string [] uriList;
+			List<string> errors;
+			
+			data = Encoding.UTF8.GetString (args.SelectionData.Data);
 			// Sometimes we get a null at the end, and it crashes us.
 			data = data.TrimEnd ('\0');
 			
-			string [] uriList = Regex.Split (data, "\r\n");
-			List<string> errors = new List<string> ();
+			errors = new List<string> ();
+			uriList = Regex.Split (data, "\r\n");
+			
 			foreach (string uri in uriList) {
-				string file;
-				string path;
+				string file, path, filename;
 				
+				if (string.IsNullOrEmpty (uri))
+					continue;
+					
 				try {
-					file = uri.Remove (0, 7);
-					string fileName = System.IO.Path.GetFileName (file);
+					file = uri.Remove (0, 7); // 7 is the length of file://
+					// I have to use System.IO here due to a Gtk namespace conflict
+					filename = System.IO.Path.GetFileName (file);
+					
 					if (!file.EndsWith (".dll")) {
-						errors.Add (fileName);
+						errors.Add (filename);
 						continue;
 					}
 
-					path = Paths.UserPluginsDirectory.Combine (fileName);
+					if (!Directory.Exists (Paths.UserAddinInstallationDirectory))
+						Directory.CreateDirectory (Paths.UserAddinInstallationDirectory);
+					
+					path = Paths.UserAddinInstallationDirectory.Combine (filename);
 					File.Copy (file, path, true);
-				} catch { }
+					
+					if (errors.Count > 0)
+						new PluginErrorDialog (errors.ToArray ());
+			
+					PluginManager.InstallLocalPlugins ();
+				} catch (Exception e) { 
+					Log<ManagePluginsPreferencesWidget>.Error ("An unexpected error occurred installing your plugin");
+					Log<ManagePluginsPreferencesWidget>.Debug ("{0}\n{1}", e.Message, e.StackTrace);
+				}
 			} 
-			
-			if (errors.Count > 0)
-				new PluginErrorDialog (errors.ToArray ());
-			
-			SetupService setup = new SetupService (AddinManager.Registry);
-			PluginManager.InstallLocalPlugins (setup);
 		}
 
 		public Bin GetConfiguration ()
@@ -147,6 +161,7 @@ namespace Do.UI
 			btn_configure.Sensitive = nview.GetSelectedAddins ()
 				.SelectMany (id => PluginManager.ConfigurablesForAddin (id))
 				.Any ();
+
 			btn_about.Sensitive = nview.GetSelectedAddins ().Any ();
 		}
 
@@ -191,14 +206,29 @@ namespace Do.UI
 			win.Modal = true;
 			win.ShowAll ();
 		}
-
-		void OnBtnAboutClicked (object sender, EventArgs e)
+		
+		void OnAboutBtnClicked (object sender, EventArgs args)
 		{
 			foreach (string id in nview.GetSelectedAddins ()) {
 				try {
-					string name = Addin.GetIdName (id).Split ('.')[1];
-					Services.Environment.OpenUrl (string.Format (PluginWikiPageFormat, name));
-				} catch { }
+					string name, url;
+					Addin a = AddinManager.Registry.GetAddin (id);
+					name = Addin.GetIdName (id).Split ('.')[1];
+					
+					// plugin manifest files support a Url attribute, if this attribute is set we should
+					// use it instead of trying to guess the wiki page.
+					if (!string.IsNullOrEmpty (a.Description.Url))
+						url = a.Description.Url;
+					if (PluginManager.PluginClassifiesAs (a, "Docklets"))
+						url = string.Format (WikiPage, name, DockletWikiPageFormat);
+					else
+						url = string.Format (WikiPage, name, PluginWikiPageFormat);
+					
+					Services.Environment.OpenUrl (url);
+				} catch (Exception e) {
+					Log.Debug (e.Message);
+					Log.Debug (e.StackTrace);
+				}
 			}
 		}
 
