@@ -37,6 +37,7 @@ namespace Do {
 		static UniverseManager universe_manager;
 
 		public static CorePreferences Preferences { get; private set; } 
+		public static CoreKeybindings Keybindings { get; private set; } 
 
 		internal static void Main (string [] args)
 		{
@@ -54,6 +55,8 @@ namespace Do {
 
 			Preferences = new CorePreferences ();
 
+			Keybindings = new CoreKeybindings ();
+
 			// Now we can set the preferred log level.
 			if (Preferences.QuietStart)
 				Log.DisplayLevel = LogLevel.Error;
@@ -77,6 +80,8 @@ namespace Do {
 				Controller.Summon ();
 			
 			Gtk.Application.Run ();
+			
+			RelevanceProvider.Serialize (RelevanceProvider.DefaultProvider);
 		}
 
 		
@@ -95,11 +100,24 @@ namespace Do {
 				return universe_manager;
 			}
 		}
+
+		static void SummonKeyCb (object sender, PreferencesChangedEventArgs e)
+		{
+			try {
+				if (e.OldValue != null)
+					keybinder.Unbind (e.OldValue as string);
+				keybinder.Bind (Keybindings.GetKeybinding ("SummonKey"), OnActivate);
+			} catch (Exception ex) {
+				Log.Error ("Could not bind summon key: {0}", ex.Message);
+				Log.Debug (ex.StackTrace);
+			}
+
+		}
 		
 		static void SetupKeybindings ()
 		{
 			try {
-				keybinder.Bind (Preferences.SummonKeybinding, OnActivate);
+				keybinder.Bind (Keybindings.GetKeybinding ("SummonKey"), OnActivate);
 			} catch (Exception e) {
 				Log.Error ("Could not bind summon key: {0}", e.Message);
 				Log.Debug (e.StackTrace);
@@ -107,16 +125,7 @@ namespace Do {
 
 			// Watch preferences for changes to the keybinding so we
 			// can change the binding when the user reassigns it.
-			Preferences.SummonKeybindingChanged += (sender, e) => {
-				try {
-					if (e.OldValue != null)
-						keybinder.Unbind (e.OldValue as string);
-					keybinder.Bind (Preferences.SummonKeybinding, OnActivate);
-				} catch (Exception ex) {
-					Log.Error ("Could not bind summon key: {0}", ex.Message);
-					Log.Debug (ex.StackTrace);
-				}
-			};
+			Keybindings.RegisterNotification ("SummonKey", SummonKeyCb);
 		}
 		
 		static void OnActivate (object sender, EventArgs e)
